/* global moment Util */
function DecodeValueFieldService(templateManager, ScreenService, maskEngine, ZhUtil) {

    var self = this;

	this.mappedFormats = {
		yy: 'YYYY',
		hh: 'hh',
		dd: 'DD',
		mm: 'MM',
		ii: 'mm',
		ss: 'ss',
		y: 'YYY',
		m: 'M',
		d: 'D',
		h: 'h',
		i: 'm',
		s: 's',
		H: 'H',
		D: 'ddd',
		M: 'MMM',
		DD: 'dddd',
		HH: 'HH',
		MM: 'MMMMM',
		a: 'a',
		A: 'A'
	};


    var DATE_FORMAT = 'mm/dd/yy';
    var TIME_FORMAT = 'hh:ii A';

    var handleDecodeType = {
        'select-native.html': decodeSelectNative,
        'radio-group.html': decodeSelectNative,
        'select-multiple.html': decodeSelectMultiple,
        'password-edit.html': passwordView,
        'password.html': passwordView,
        'phone.html': phoneView,
        'checkbox.html': decodeCheckboxValue,
        'datetime.html': decodeDateTime,
        'calendar.html': decodeCalendar,
        'select-month.html': decodeSelectMonth,
        'interval.html': decodeInterval,
        'default': dafaultTemplate
    };

    var handleTemplateType = {
        'image.html': 'image',
        'image-static.html': 'image'
    };

    var handleCustomViewTemplate = {};

    this.getRegistered = function () {
        return handleDecodeType;
    };

    this.register = function register(template, callback) {
        handleDecodeType[template] = callback;
    };

    this.registerType = function (template, type) {
        if (handleTemplateType.hasOwnProperty(template)) {
            throw new Error("There is already a type registered for this template.");
        }
        handleTemplateType[template] = type;
    };

    this.registerCustomViewTemplate = function (template, viewTemplate) {
        if (handleCustomViewTemplate.hasOwnProperty(template)) {
            throw new Error("There is already a view template registered for this template.");
        }
        handleCustomViewTemplate[template] = viewTemplate;
    };

    this.decodeViewTemplate = function (decode, field) {
        var template = decode.isLabel(field) ? 'field/label.html' : 'field/span-field.html';
        if (handleCustomViewTemplate.hasOwnProperty(field.template)) {
            template = handleCustomViewTemplate[field.template];
        }
        return templateManager.getTemplateURL(template);
    };

    this.getFieldType = function (field) {
        return handleTemplateType[field.getTemplateSimpleName()];
    };

    this.model = function (field, fieldRow) {
        var viewRow;
        if (field) {
            var template = field.getTemplateSimpleName();
            if (handleDecodeType.hasOwnProperty(template)) {
                viewRow = handleDecodeType[template](field, fieldRow);
            } else {
                viewRow = handleDecodeType.default(field, fieldRow);
            }
        }
        return field.translate ? ScreenService.i18n(viewRow) : viewRow;
    };

    function dafaultTemplate(field, fieldRow) {
        if (field.mask)
            return Util.applyMask(fieldRow, field.mask, maskEngine, ZhUtil);
        else if (field.options)
            return decodeMultipleOptions(fieldRow, field.options);
        else
            return fieldRow;
    }

    function passwordView(field, fieldRow) {
        if (fieldRow)
            return fieldRow.replace(/./g, "*");
    }

    function phoneView(field, fieldRow) {
        var finalNum = "";

        if (fieldRow) {
            fieldRow = fieldRow.replace(/[^\d]/g, '');

            finalNum += '(' + fieldRow.slice(0, 2) + ') ';

            if (fieldRow.length === 10) {
                finalNum += fieldRow.slice(2, 6) + '-';
                finalNum += fieldRow.slice(6, 10);
            } else if (fieldRow.length === 11) {
                finalNum += fieldRow.slice(2, 7) + '-';
                finalNum += fieldRow.slice(7, 11);
            }
        }
        return finalNum;
    }

    function decodeSelectMultiple(field, fieldRow) {
        var viewRow = "";
        var matchDataSource;
        if (field && fieldRow && field.dataSource && field.dataSource.data && !field.dataSource.allToggled) {
            if (field.valueFields) {
                var previouslyMatched = {};
                matchDataSource = function (item) {
                    return fieldRow.some(function (row, index) {
                        if (previouslyMatched[index]) {
                            return false;
                        } else {
                            var matches = field.valueFields.every(function (valueField) {
                                return row[valueField] == item[valueField];
                            });
                            if (matches) {
                                previouslyMatched[index] = true;
                            }
                            return matches;
                        }
                    });
                };
            } else {
                matchDataSource = function (item) {
                    return fieldRow.indexOf(item[field.valueField]) !== -1;
                };
            }
            viewRow = field.dataSource.data
                .filter(function (item) {
                    return matchDataSource(item);
                })
                .map(function (item) {
                    return item[field.descriptionField];
                })
                .join(", ");
        }
        return viewRow;
    }

    function decodeSelectNative(field, fieldRow) {
        var viewRow = "";

        if (field && fieldRow && field.dataSource) {
            var found = false;
            var data = field.dataSource.data;

            for (var i = 0; i < data.length && !found; i++) {
                if (data[i][field.valueField] === fieldRow) {
                    viewRow = data[i][field.descriptionField];
                    found = true;
                }
            }
        }

        return viewRow;
    }

    function decodeCheckboxValue(field, fieldRow) {
        var viewRow = "";
        if (field) {
            viewRow = ScreenService.i18n((fieldRow === field.valueToTrue ? (field.labelToTrue || field.label) : field.labelToFalse));
        }
        return viewRow;
    }

    function resolveDateTimeFormat(field) {
        var format;
        var dateFormat = field['datetime-dateFormat'] || DATE_FORMAT;
        var timeFormat = field['datetime-timeFormat'] || TIME_FORMAT;

        switch (field['datetime-preset']) {
            case 'date':
            case 'month':
                format = dateFormat;
                break;
            case 'time':
                format = timeFormat;
                break;
            default:
                format = dateFormat + ' ' + timeFormat;
        }

        return format;
    }

	this.translateFormats = function (input) {
	    var separator = input.match(/\W/);
	    if(separator){
	        separator = separator[0];
    		return (input.split(separator) || []).map(function (pieceFormat) {
    			return self.mappedFormats[pieceFormat];
    		}).join(separator);
	    } else {
	        return input;
	    }
	};

	function resolveDateTimeDisplayFormat(format, fieldRow, displayFormat) {
		var formatsArr = format.split(" ");
		formatsArr.forEach(function (format, index) {
			var timeFormat = format.match(/[h,i,s,a]/i);
			if (timeFormat) {
				formatsArr[index] = self.translateFormats(timeFormat.input);
			} else {
				var dateFormat = format.match(/[d,m,y]/i);
				if (dateFormat) {
					formatsArr[index] = self.translateFormats(dateFormat.input);
				}
			}
		});
		format = formatsArr.join(" ");
		return moment(fieldRow, format).format(displayFormat || format);
	}


	function resolveDisplayFormat(format, fieldRow, displayFormat) {
		return moment(fieldRow, format).format(displayFormat || format);
	}

    function decodeDateTime(field, fieldRow) {
        var viewRow = "";
        if (field && fieldRow) {
            viewRow = resolveDateTimeDisplayFormat(resolveDateTimeFormat(field), fieldRow, field.displayFormat);
        }
        return viewRow;
    }

    function decodeCalendar(field, fieldRow) {
        var viewRow = "";
        if (field && fieldRow) {
            viewRow = resolveDisplayFormat(field.dateFormat, fieldRow, field.displayFormat);
        }
        return viewRow;
    }

    function decodeInterval(field, fieldRow) {
        var viewRow = "";

        if (field && fieldRow) {
            if (fieldRow.start) {
                viewRow += ScreenService.i18n('From') + " " + fieldRow.start + " ";
            }

            if (fieldRow.end) {
                viewRow += ScreenService.i18n('to') + " " + fieldRow.end;
            }
        }

        return viewRow;
    }

    function decodeSelectMonth(field, fieldRow) {
        var viewRow = "";
        if (field && fieldRow) {
            viewRow = resolveDisplayFormat(field.dateFormat, fieldRow, field.displayFormat);
        }
        return viewRow;
    }

    function decodeMultipleOptions(fieldRow, options) {
        if (options.hasOwnProperty(fieldRow)) {
            return ScreenService.i18n(options[fieldRow]);
        } else {
            return fieldRow;
        }
    }
}

var DecodeValueField = function ($scope, ScreenService, maskEngine, ZhUtil, DecodeValueFieldService, decodeTemplateFieldService) {
    this.isFieldTemplate = function (field, template) {
        return field.template && field.template.indexOf("field/" + template) !== -1;
    };

    this.isColor = function (field) {
        return this.isFieldTemplate(field, "color.html");
    };

    this.isCheckbox = function (field) {
        return this.isFieldTemplate(field, "checkbox.html");
    };

    this.isButton = function (field) {
        return this.isFieldTemplate(field, "button.html");
    };

    this.isLabelWithIcon = function (field) {
        return this.isFieldTemplate(field, "label-with-icon.html");
    };

    this.isTextarea = function (field) {
        return this.isFieldTemplate(field, "textarea.html");
    };

    this.isRadioGroup = function (field) {
        return this.isFieldTemplate(field, "radio-group.html");
    };

    this.isImage = function (field) {
        return this.isFieldTemplate(field, "image.html");
    };

    this.isLabel = function (field) {
        return this.isFieldTemplate(field, "label.html");
    };

    this.isCalendar = function (field) {
        return this.isFieldTemplate(field, "calendar.html");
    };

    this.isSelectMonth = function (field) {
        return this.isFieldTemplate(field, "select-month.html");
    };

    this.isInterval = function (field) {
        return this.isFieldTemplate(field, "interval.html");
    };

    this.isLabelStaticType = function (field) {
        return this.isFieldTemplate(field, "label-static.html");
    };

    this.isSelectMultiple = function (field) {
        return this.isFieldTemplate(field, "select-multiple.html");
    };

    this.model = function (field, fieldRow) {
        return DecodeValueFieldService.model(field, fieldRow);
    };

    this.isFieldImageType = function (field) {
        return DecodeValueFieldService.getFieldType(field) === 'image';
    };

    this.getViewTemplate = function (field) {
        var viewTemplate = this.isLabel(field) ? 'field/label.html' : 'field/span-field.html';
        return decodeTemplateFieldService.decodeViewTemplate(field, viewTemplate);
    };
};